<?php
// $Id: nodehierarchy.views.inc,v 1.1.2.2 2008/05/14 22:22:00 ronan Exp $

// Implementations of Views Hooks
/**
 * Implementation of hook_views_data()
 */
function nodehierarchy_views_views_data() {
  // Basic table information.

  // Define the base group of this table. Fields that don't
  // have a group defined will go into this field by default.
  $data['nodehierarchy']['table']['group']  = t('Node Hierarchy');

  // Joins
  $data['nodehierarchy']['table']['join'] = array(
    //...to the node table
    'node' => array(
      'handler' => 'views_join',
      'arguments' => array('nodehierarchy', 'node', 'nid', 'nid'),
    ),
  );

  // ----------------------------------------------------------------
  // Fields.

  // Parent.
  $data['nodehierarchy']['parent'] = array(
    'title' => t('Parent Node'),
    'help' => t('The parent id of the node.'),
    // Information for displaying the parent.
    'field' => array(
      'handler' => 'views_handler_field_nodehierarchy_parent',
      'click sortable' => TRUE,
    ),
    // Information for accepting a parent as an argument.
    'argument' => array(
      'handler' => 'views_handler_argument_node_nid',
      'click sortable' => TRUE,
    ),
    // Information for accepting a parent as a filter.
    'filter' => array(
      'handler' => 'views_handler_filter_equality',
    ),

  );

  // Sort order.
  $data['nodehierarchy']['order_by'] = array(
    'title' => t('Child Sort Order'),
    'help' => t('The assigned order of the node relative to it\'s siblings.'),
    // Information for sorting on a order_by.
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  // Bogus fields for aliasing purposes.
  $data['nodehierarchy']['antecedent'] = array(
    'title' => t('Ancestor Node ID'),
    'help' => t('Use this if you want to display any descendant of the given node'),
    'argument' => array(
      'handler' => 'views_handler_argument_nodehierarchy_ancestor',
    ),
  );

  $data['nodehierarchy']['actions'] = array(
    'title' => t('Actions'),
    'help' => t('Edit, view and reorder actions'),
    'field' => array(
      'field' => 'nid', // the real field
      'group' => t('Node Hierarchy'), // The group it appears in on the UI,
      'handler' => 'views_handler_field_nodehierarchy_actions',
      'click sortable' => FALSE,
    ),
  );

  return $data;
}

// Handlers.
/**
 * Argument handler to accept a node type.
 *
 * @ingroup views_argument_handlers
 */
class views_handler_argument_nodehierarchy_ancestor extends views_handler_argument {

  /**
   * Build the query based upon the formula
   */
  function query() {
    $this->ensure_my_table();
    $descendants = _nodehierarchy_views_get_descendant_parents($this->argument);
    $descendant_list = implode(",", $descendants);
    $this->query->add_where(0, "nodehierarchy.parent IN ( $descendant_list )");
  }
}

/**
 * Return the list of descendants of the given node which can also themselves be parents.
 */
function _nodehierarchy_views_get_descendant_parents($nid) {
  static $parent_types = NULL;
  if (!$parent_types) {
    $types = _nodehierarchy_get_parent_types("", TRUE);
    $parent_types = implode(",", $types);
  }

  $out = array($nid);
  $result = db_query("SELECT n.nid FROM {node} n LEFT JOIN {nodehierarchy} h ON h.nid = n.nid WHERE h.parent = %d and n.type IN (". $parent_types .")", $nid);
  while ($child = db_fetch_object($result)) {
    $out = array_merge($out, _nodehierarchy_views_get_descendant_parents($child->nid));
  }

  return $out;
}

/**
 * Field handler to allow linking to a comment
 *
 * @ingroup views_field_handlers
 */
class views_handler_field_nodehierarchy_parent extends views_handler_field {
  /**
   * Override init function to provide generic option to link to comment.
   */
  function init(&$view, &$data) {
    parent::init($view, $data);
    if (isset($data['link_to_parent']) && $view->base_table != 'node') {
      // @todo: This can't be right, it'll totally fail with relationships.
      $this->additional_fields[] = 'nid';
      $this->nid_field = 'node_nid';
    }
    else {
      $this->nid_field = 'parent';
    }
  }

  function options(&$options) {
    parent::options($options);
    $options['link_to_parent'] = TRUE;
    $options['display_as'] = 'title';
  }

  /**
   * Provide link-to-comment option.
   */
  function options_form(&$form, &$form_state) {
    $form['display_as'] = array(
      '#title' => t('Display this field as'),
      '#type' => 'select',
      '#options' => array('title' => t('Node Parent Title'), 'nid' => t('Node Parent ID')),
      '#default_value' => $this->options['display_as'],
    );
    $form['link_to_parent'] = array(
      '#title' => t('Link this field to the parent node'),
      '#type' => 'checkbox',
      '#default_value' => $this->options['link_to_parent'],
    );
  }

  function render_parent($data, $values) {
    $out = "";
    if (@$this->options['display_as'] === 'nid') {
      $out = $data;
    }
    else {
      $parent = node_load($data);
      $out = $parent->title;
    }
    if (!empty($this->options['link_to_parent'])) {
      return l($out, "node/". $data );
    }
    else {
      return $out;
    }
  }

  function render($values) {
    return $this->render_parent(check_plain($values->{$this->field_alias}), $values);
  }
}

/**
 * Field handler to allow linking to a comment.
 *
 * @ingroup views_field_handlers
 */
class views_handler_field_nodehierarchy_actions extends views_handler_field {
  function render($values) {
    $node = new stdClass();
    $node->nid = $values->{$this->field_alias};
    return theme("nodehierarchy_actions", $node);
  }
}